==> keystone_sdk-0.0.5-3.tar.gz: keystone_sdk-0.0.5-3/keystone/app_client/drain.py <==
# -*- coding: utf-8 -*-

from datetime import datetime
import pprint
from Queue import Queue
import thread
import sys
import time
import zmq

import keystone.app_client.zmq_utils as zmq_utils
import keystone.parameters
import keystone.utils
from keystone.api import keystone_class, api_method
from keystone.coordinator import KSObserver, KSCoordinator
from keystone.app_client.session import Session
from keystone.app_client.json_utils import json_clean, UTC_TIMESTAMP
from keystone.order import KSOrderEventType

class Drain(KSObserver):
	class DrainThreadStop:
		pass
	session = Session()
	context = zmq.Context()
	def __init__(self, 
		strategy_manager, 
		drain_url='tcp://127.0.0.1:55558', 
		syn_url='tcp://127.0.0.1:55559', 
		debug=False,
		send_data=False):
		self.strategy_manager = strategy_manager
		self.drain_url = drain_url
		self.syn_url = syn_url
		self.debug = debug
		self.send_data = send_data
		self.orders = []
		self.buffer = Queue()
		self.sock = zmq_utils.create_publisher_socket(self.drain_url, self.syn_url)

	def msg(self, dt):
		msg = {
		'datetime': dt,
		'portfolio_value': self.strategy_manager.portfolio.value(),
		'positions': [pos.to_dict() for pos in self.strategy_manager.portfolio.positions()],
		'orders': [order.to_dict() for order in self.orders],
		}
		msg.update(self.strategy_manager.analyzers[0].value())
		return msg

	def onDataEvent(self, dataEvent):
		# generate message
		msg = self.msg(dataEvent.time())
		# msg = keystone.utils.squash_dates(msg)
		msg = json_clean(msg, datetime_format = UTC_TIMESTAMP)
		if self.send_data:
			msg['data'] = dataEvent.getAllData()
		# clear orders
		self.orders = []
		# trigger send method
		self.send(msg)

	def onOrderEvent(self, orderEvent):
	    if orderEvent.type != KSOrderEventType.ACCEPTED and orderEvent.type != KSOrderEventType.CANCELLED:
	    	self.orders.append(orderEvent.txn)

	def send(self, msg):
		self.buffer.put(msg, block = True)
		# self.session.send(self.sock, u'drain', content = message, ident = 'stream.drain')

	def sendMessage(self):
		count = 1
		while True:
			message = self.buffer.get(block = True)
			if isinstance(message, self.DrainThreadStop):
				return
			self.session.send(self.sock, u'drain', content = message, ident = 'stream.drain')
			# print "send %d"%(count)
			# count += 1

	def start(self):
		thread.start_new_thread(self.sendMessage, ())

	def wait(self):
		self.buffer.put(self.DrainThreadStop())

==> keystone_sdk-0.0.5-5.tar.gz: keystone_sdk-0.0.5-5/keystone/app_client/drain.py <==
# -*- coding: utf-8 -*-

from datetime import datetime
import pprint
from Queue import Queue
import thread
import sys
import time
import zmq

import keystone.app_client.zmq_utils as zmq_utils
import keystone.parameters
import keystone.utils
from keystone.api import keystone_class, api_method
from keystone.coordinator import KSObserver, KSCoordinator
from keystone.app_client.session import Session
from keystone.app_client.json_utils import json_clean, UTC_TIMESTAMP
from keystone.order import KSOrderEventType

class Drain(KSObserver):
	class DrainThreadStop:
		pass
	session = Session()
	context = zmq.Context()
	def __init__(self, 
		strategy_manager, 
		drain_url='tcp://127.0.0.1:55558', 
		syn_url='tcp://127.0.0.1:55559', 
		debug=False,
		send_data=False):
		self.strategy_manager = strategy_manager
		self.drain_url = drain_url
		self.syn_url = syn_url
		self.debug = debug
		self.send_data = send_data
		self.orders = []
		self.buffer = Queue()
		self.sock = zmq_utils.create_publisher_socket(self.drain_url, self.syn_url)

	def msg(self, dt):
		msg = {
		'datetime': dt,
		'portfolio_value': self.strategy_manager.portfolio.value(),
		'positions': [pos.to_dict() for pos in self.strategy_manager.portfolio.positions()],
		'orders': [order.to_dict() for order in self.orders],
		}
		msg.update(self.strategy_manager.analyzers[0].value())
		return msg

	def onDataEvent(self, dataEvent):
		# generate message
		msg = self.msg(dataEvent.time())
		# msg = keystone.utils.squash_dates(msg)
		msg = json_clean(msg, datetime_format = UTC_TIMESTAMP)
		if self.send_data:
			msg['data'] = dataEvent.getAllData()
		# clear orders
		self.orders = []
		# trigger send method
		self.send(msg)

	def onOrderEvent(self, orderEvent):
	    if orderEvent.type != KSOrderEventType.ACCEPTED and orderEvent.type != KSOrderEventType.CANCELLED:
	    	self.orders.append(orderEvent.txn)

	def send(self, msg):
		# self.buffer.put(msg, block = True)
		self.session.send(self.sock, u'drain', content = msg, ident = 'stream.drain')

	def sendMessage(self):
		count = 1
		while True:
			message = self.buffer.get(block = True)
			if isinstance(message, self.DrainThreadStop):
				return
			self.session.send(self.sock, u'drain', content = message, ident = 'stream.drain')
			# print "send %d"%(count)
			# count += 1

	def start(self):
		return
		thread.start_new_thread(self.sendMessage, ())

	def wait(self):
		self.buffer.put(self.DrainThreadStop())
